function [integral,xinf,xsup]=oscintegral(OscRecord,wfm,x1,x2,shouldPlot=false)

# [integral, xinf, xsup] = oscintegral(OscRecord, WaveformInt, x1, x2, shouldPlot)
#
# Calculates the integral of the waveform WaveformInt over its x-axis (as contained 
# in the structure OscRecord) from the point x1 to the point x2. The exact  
# integration limits, considering the discrete character of the 
# x-axis, are also returned as xinf and xsup.
#
# If the optional parameter shouldPlot is true, then the operation is illustrated 
# in a graph with the area under the waveform WaveformInt that corresponds to the 
# integral.
#
# If WaveformInt is a number, it is treated as the index of the waveform within 
# OscRecord. Otherwise, it is treated as its label as set in the oscilloscope.
  
# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology

  if (!isnumeric(wfm))
    wfm = oscwfmidx(OscRecord, wfm);
  endif
  
  WaveformInt = OscRecord.Waveforms(wfm);
  [i1,xinf] = oscsearchx(WaveformInt, x1);
  [i2,xsup] = oscsearchx(WaveformInt, x2);
  xVals = WaveformInt.Values.x(i1:i2);
  yVals = WaveformInt.Values.y(i1:i2);
  integral = trapz(xVals, yVals);
  
  if (shouldPlot)
    figure();
    oscplotwfm(OscRecord, wfm, 0, 1, 0, "xy");
    hold on;
    area([xVals xVals(numel(xVals)) xVals(1)], [yVals 0 0]);
    hax=gca();
    yLims = get(hax, "ylim");
    plot([xinf xinf], yLims, ":r");
    plot([xsup xsup], yLims, ":r");
    hold off;    
  endif
  
endfunction
