function [integral,x1,x2]=oscintegral3(OscRecord,wfmInt,wfm1,y1,wfm2,y2,shouldPlot=false)

# [integral, x1, x2] = oscintegral3(OscRecord, ...
# wfmInt, wfm1, y1, wfm2, y2, shouldPlot)
#
# Calculates the integral of the waveform wfmInt over its x-axis (as contained 
# in the structure OscRecord) from a point x1 where the waveform wfm1 first 
# crosses the level y1 to a point x2 where the waveform wfm2 subsequently crosses 
# the level y2.
#
# If the optional parameter shouldPlot is true, then the operation is illustrated 
# in a graph with three sub-plots, with the first one visualises the point 
# (x1, y1) on the waveform wfm1; the second one, the point (x2, y2) on the 
# waveform wfm2; and the third one, the area under the waveform wfmInt 
# corresponding to the integral.
#
# If wfm* is a number, it is treated as the index of the respective waveform 
# within OscRecord. Otherwise, it is treated as its label as set in the 
# oscilloscope.
  
# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology

  if (!isnumeric(wfmInt))
    wfmInt = oscwfmidx(OscRecord, wfmInt);
  endif
  if (!isnumeric(wfm1))
    wfm1 = oscwfmidx(OscRecord, wfm1);
  endif
  if (!isnumeric(wfm2))
    wfm2 = oscwfmidx(OscRecord, wfm2);
  endif

  [i1,x1] = oscsearchy(OscRecord.Waveforms(wfm1), y1);
  [i2,x2] = oscsearchy(OscRecord.Waveforms(wfm2), y2, x1);
  WaveformInt = OscRecord.Waveforms(wfmInt);
  xVals = WaveformInt.Values.x(i1:i2);
  yVals = WaveformInt.Values.y(i1:i2);
  integral = trapz(xVals, yVals);
  
  if (shouldPlot)
    ys = [y1 y2];
    xs = [x1 x2];
    wfms = [wfm1 wfm2];
    figure();
    for iWfm = [1:2]    
      hax=subplot(3,1,iWfm);
      oscplotwfm(OscRecord, wfms(iWfm), 0, 1, 0, "xy");
      hold on;
      xLims = get(hax, "xlim");
      plot([xLims(1) xLims(2)], [ys(iWfm) ys(iWfm)], ":r");
      yLims = get(hax, "ylim");
      plot([x1 x1], yLims, ":r");
      plot([x2 x2], yLims, ":r");
      plot(xs(iWfm), ys(iWfm), "+r", "markersize", 8, "linewidth", 1);
      hold off;
    endfor
    hax=subplot(3,1,3);
    oscplotwfm(OscRecord, wfmInt, 0, 1, 0, "xy");
    hold on;
    area([xVals xVals(numel(xVals)) xVals(1)], [yVals 0 0]);
    yLims = get(hax, "ylim");
    plot([x1 x1], yLims, ":r");
    plot([x2 x2], yLims, ":r");
    hold off;    
  endif
  
endfunction
